'use strict';

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var HTTP = 'http:',
    HTTPS = 'https:',
    Resolver = require('trucks-resolver-core'),
    url = require('url'),
    types = ['text/html', 'application/javascript', 'text/css'];

/**
 *  Resolve `http:` and `https:` protocols.
 *
 *  @public {class} HttpResolver
 */

var HttpResolver = function (_Resolver) {
  _inherits(HttpResolver, _Resolver);

  /**
   *  Create an HTTP resolver.
   *
   *  @public {constructor} HttpResolver
   */

  function HttpResolver() {
    _classCallCheck(this, HttpResolver);

    // include default port when getting canonical path

    var _this = _possibleConstructorReturn(this, Object.getPrototypeOf(HttpResolver).apply(this, arguments));

    _this.getDefaultPort(_this.uri.protocol, _this.uri);
    return _this;
  }

  /**
   *  Determine if a response content type can be processed.
   *
   *  @private {function} isValidContentType
   *  @param {String} type the response content type.
   *
   *  @returns Boolean indicating whether the content type is acceptable.
   */


  _createClass(HttpResolver, [{
    key: 'isValidContentType',
    value: function isValidContentType(type) {
      var i = 0;
      for (; i < types.length; i++) {
        if (~type.indexOf(types[i])) {
          return true;
        }
      }
      return false;
    }

    /**
     *  Loads a remote HTTP resource from the network and invokes 
     *  callback with the response body.
     *
     *  If the response has a `Content-Encoding` containing gzip it is deflated.
     *
     *  @public {function} resolve
     *  @param {Function} cb callback function.
     */

  }, {
    key: 'resolve',
    value: function resolve(cb) {
      var _this2 = this;

      var zlib = require('zlib'),
          http = require('http');

      var called = false;

      function done(err, buf) {
        /* istanbul ignore next: guard against multiple events firing */
        if (called) {
          return;
        }
        cb(err, buf);
        called = true;
      }

      // ensure we are resolved relative to a parent
      this.uri = url.parse(this.getCanonicalPath());

      if (!this.uri.protocol) {
        return cb(new Error('resolver attempt to load with no protocol'));
      }

      var pth = this.uri.pathname;

      var uri = url.format(this.uri),
          options = {
        hostname: this.uri.hostname,
        port: this.uri.port,
        path: pth,
        headers: {
          'Accept': types.join(', ')
        }
      };

      this.getDefaultPort(this.uri.protocol, options);

      var req = http.get(options, function (res) {
        // expecting 200 response
        if (res.statusCode !== 200) {
          return done(new Error('unexpected status code ' + res.statusCode + ' from ' + uri));
        }

        var contentType = res.headers['content-type'],
            encoding = res.headers['content-encoding'];

        var gzip = false;

        if (!_this2.isValidContentType(contentType)) {
          return done(new Error('unexpected content type ' + contentType));
        }

        if (encoding && ~encoding.indexOf('gzip')) {
          gzip = true;
        }

        var buf = new Buffer(0);

        // consume response body
        res.resume();

        res.on('data', function (chunk) {
          buf = Buffer.concat([buf, chunk], buf.length + chunk.length);
        });

        // handle response error
        res.once('error', function (err) {
          /* istanbul ignore next: tough to mock repsonse stream error */
          done(err);
        });

        res.once('end', function () {
          if (gzip) {
            zlib.gunzip(buf, function (err, contents) {
              /* istanbul ignore next: not going to mock zlib deflate error */
              if (err) {
                return done(err);
              }
              done(null, contents);
            });
          } else {
            done(null, buf);
          }
        });
      });

      // handle request error
      req.once('error', function (err) {
        done(err);
      });
    }

    /**
     *  Get a canonical path for the URL reference, used to determine if the 
     *  resource has already been processed.
     */

  }, {
    key: 'getCanonicalPath',
    value: function getCanonicalPath() {
      // no scheme with a parent, resolve relative to the parent
      if (!this.uri.protocol && this.parent && this.parent.file) {
        return url.resolve(this.parent.file, this.href);
      }

      // should be an absolute HTTP/HTTPS URL
      return url.format(this.uri);
    }

    /**
     *  Injects a default port for the `http:` and `https:` protocols.
     *
     *  Used for requests and generating canonical URLs.
     *
     *  @private {function} getDefaultPort
     *  @param {String} protocol parsed protocol.
     *  @param {Object} target the object to receive the `port` property.
     *
     *  @returns the input target object.
     */

  }, {
    key: 'getDefaultPort',
    value: function getDefaultPort(protocol, target) {
      if (!target.port) {
        if (protocol === HTTP) {
          target.port = 80;
        } else {
          target.port = 443;
        }
      }
      return target;
    }
  }]);

  return HttpResolver;
}(Resolver);

HttpResolver.HTTP = HTTP;
HttpResolver.HTTPS = HTTPS;

/**
 *  Plugin for the http resolver.
 *
 *  Registers the resolver class for the `http:` and `https:` protocols unless 
 *  the `secure` option is given in which case the `http:` protocol is not 
 *  registered and attempts to use `http:` URLs in HTML imports will generate 
 *  errors.
 *
 *  @public {function} http
 *  @param {Object} state compiler state.
 *  @param {Object} conf plugin configuration object.
 *  @option {Boolean=false} secure only use `https:`.
 */
function http(state, conf) {
  return function (registry) {
    if (!conf.secure) {
      registry.register(HTTP, HttpResolver);
    }
    registry.register(HTTPS, HttpResolver);
  };
}

http.Resolver = HttpResolver;

module.exports = http;